# CSS Color Module Level 5

## Editor’s Draft, 1 January 1970

Specification Metadata

This version:
  https://drafts.csswg.org/css-color-5/
Latest published version:
  https://www.w3.org/TR/css-color-5/
Editors:
  Chris Lilley (W3C)
  Una Kravets (Google)
  Lea Verou (Invited Expert)
  Adam Argyle (Google)
Issue Tracking:
  GitHub Issues
Suggest an Edit for this Spec:
  GitHub Editor

Copyright © 1970 W3C® (MIT, ERCIM, Keio, Beihang). W3C liability, trademark and permissive document license rules apply.

## Abstract

This module extends CSS Color [css-color-4] to add color modification functions.

CSS is a language for describing the rendering of structured documents (such as HTML and XML) on screen, on paper, etc.

## Status of this document

This is a public copy of the editors’ draft. It is provided for discussion only and may change at any moment. Its publication here does not imply endorsement of its contents by W3C. Don’t cite this document other than as work in progress.

GitHub Issues are preferred for discussion of this specification. When filing an issue, please put the text “css-color” in the title, preferably like this: “[css-color] …summary of comment…”. All issues and comments are archived, and there is also a historical archive.

This document was produced by the CSS Working Group.

This document was produced by a group operating under the W3C Patent Policy. W3C maintains a public list of any patent disclosures made in connection with the deliverables of the group; that page also includes instructions for disclosing a patent. An individual who has actual knowledge of a patent which the individual believes contains Essential Claim(s) must disclose the information in accordance with section 6 of the W3C Patent Policy.

This document is governed by the 1 March 2019 W3C Process Document.

## 1. Introduction

This section is not normative.

Web developers, design tools and design system developers often use color functions to assist in scaling the design of their component color relations. With the increasing usage of design systems that support multiple platforms and multiple user preferences, like the increased capability of Dark Mode in UI, this becomes even more useful to not need to manually set color, and to instead have a single source from which schemes are calculated.

Currently Sass, calc() on HSL values, or PostCSS is used to do this. Preprocessors are unable to work on dynamically adjusted colors, all current solutions are restricted to the sRGB gamut and to the perceptual limitations of HSL (colors are bunched up in the color wheel, and two colors with visually different lightness, like yellow and blue, can have the same HSL lightness).

This module adds three functions: color-mix, color-contrast, and a way to modify colors.

The perceptually uniform ``lch()`` colorspace is used for mixing by default, as this has no gamut restrictions and colors are evenly distributed. However, other colorspaces can be specified, including ``hsl()`` or ``srgb`` if desired.

## 2. Mixing colors: the color-mix function

This function takes two <color> specifications and returns the result of mixing them, in a given colorspace, by a specified amount.

Unless otherwise specified, the mixing is done in the lch() colorspace.

Multiple color functions can be specified.

    color-mix() = color-mix( <color>  <color> [ <number> | <percentage> | [ <color-function> <colorspace>? ]?] )

This example produces the mixture of red and white, in lch() colorspace (the default), with the lightness being 50% of the lightness of red (and thus, 50% of the lightness of white). The chroma and hue of red are left unchanged.

    mix-color(red, white, lightness(50%));

The calculation is as follows:

- sRGB red (#F00) is lch(54.2917 106.8390 40.8526)

- sRGB white (#FFF) is lch(100 0 0)

- mix lightness is 54.2917 * 0.5 + 100 * 0.5 = 77.14585

- mixed result is lch(77.14585 81.95 37.192)

- which results in a lighter red, like a Salmon color

This example produces the mixture of red and yellow, in lch() colorspace (the default), with the lightness being 30% of the lightness of red (and thus, 70% of the lightness of yellow). The chroma and hue of red are left unchanged.

    mix-color(red, yellow, lightness(30%));

The calculation is as follows:

- sRGB red (#F00) is lch(54.2917 106.8390 40.8526)

- sRGB yellow (#FF0) is lch(97.6071 94.7077 99.5746)

- mix lightness is 54.2917 * 0.3 + 97.6071 * 0.7 = 84.6125

- mixed result is lch(84.6125 106.8390 40.8526)

- which is a very light red (and outside the gamut of sRGB: rgb(140.4967% 51.2654% 32.6891%))

Instead of a list of color functions, a plain number or percentage can be specified, which applies to all color channels.

This example produces the mixture of red and yellow, in lch colorspace (the default), with each lch channel being 65% of the value for red and 35% of the value for yellow.

Note: interpolating on hue and chroma keeps the intermediate colors as saturated as the endpoint colors.

    mix-color(red, yellow, 65%);

The calculation is as follows:

- sRGB red (#F00) is lch(54.2917 106.8390 40.8526)

- sRGB yellow (#FF0) is lch(97.6071 94.7077 99.5746)

- mix lightness is 54.2917 * 0.65 + 97.6071 * 0.35 = 69.4521

- mix chroma is 106.83 * 0.65 + 94.7077 * 0.35 = 102.5872

- mix hue is 40.8526 * 0.65 + 99.5746 * 0.35 = 61.4053

- mixed result is lch(69.4521 102.5872 61.4053)

- which is a red-orange: rgb(75.3600% 65.6304% 16.9796%)

## 3. Selecting the most contrasting color: the color-contrast() function

This function takes, firstly, a single color (typically a background, but not necessarily), and then second, a list of colors; it selects from that list the color with highest luminance contrast to the single color.

    color-contrast(wheat tan, sienna, var(--myAccent), #d2691e)

The calculation is as follows:

- wheat (#f5deb3), the background, has relative luminance 0.749

- tan (#d2b48c) has relative luminance 0.482 and contrast ratio 1.501

- sienna (#a0522d) has relative luminance 0.137 and contrast ratio 4.273

- suppose myAccent has the value #b22222

- #b22222 has relative luminance 0.107 and contrast ratio 5.081

- #d2691e has relative luminance 0.305 and contrast ratio 2.249

- The highest contrast ratio is 5.081 so var(--myAccent) wins

## 4. Modifying colors

Note: There are currently two proposals for modifying colors: color-adjust and Relative color syntax.

there are two proposals for color modification (proposal 1, proposal 2). The CSS WG expects that the best aspects of each will be chosen to produce a single eventual solution. <https://github.com/w3c/csswg-drafts/issues/3187>

### 4.1. Adjusting colors: the color-adjust function

This function takes one <color> specification and returns the result of adjusting that color, in a given colorspace, by a specified transform function.

Unless otherwise specified, the adjustment is done in the lch() colorspace.

Multiple color functions can be specified.

    color-adjust() = color-adjust( <color> [ <color-function> <colorspace>? ]?] )

This example produces the adjustment of peru (#CD853F), in lch() colorspace (the default), with the lightness being reduced by 20%. The chroma and hue of red are left unchanged.

    adjust-color(peru, lightness(-20%));

The calculation is as follows:

- peru (#CD853F) is lch(62.2532% 54.0114 63.6769)

- adjusted lightness is 62.2532% - 20% = 42.2532%

- adjusted result is lch(42.2532% 54.0114 63.6769)

- which is rgb(57.58%, 32.47%, 3.82%)

### 4.2. Relative color syntax

Besides specifying absolute coordinates, all color functions can also be used with a *relative syntax* to produce colors in the function’s target color space, based on an existing color (henceforth referred to as "origin color"). This syntax consists of the keyword from, a <color> value, and optionally numerical coordinates specific to the color function. To allow calculations on the original color’s coordinates, there are single-letter keywords for each coordinate and `alpha` that corresponds to the color’s alpha. If no coordinates are specified, the function merely converts the origin color to the target function’s color space.

The following sections outline the relative color syntax for each color function.

A future version of this specification may define a relative syntax for color() as well.

#### 4.2.1. Relative RGB colors

The grammar of the rgb() function is extended as follows:

    rgb() = rgb([from <color>]? <percentage>{3} [ / <alpha-value> ]? ) |
        rgb([from <color>]? <number>{3} [ / <alpha-value> ]? )
    <alpha-value> = <number> | <percentage>

When an origin color is present, the following keywords can also be used in this function (provided the end result conforms to the expected type for the parameter) and correspond to:

- r is a <percentage> that corresponds to the origin color’s red channel after its conversion to sRGB

- g is a <percentage> that corresponds to the origin color’s green channel after its conversion to sRGB

- b is a <percentage> that corresponds to the origin color’s blue channel after its conversion to sRGB

- alpha is a <percentage> that corresponds to the origin color’s alpha transparency

#### 4.2.2. Relative HSL colors

The grammar of the hsl() function is extended as follows:

    hsl() = hsl([from <color>]? <hue> <percentage> <percentage> [ / <alpha-value> ]? )
    <hue> = <number> | <angle>

When an origin color is present, the following keywords can also be used in this function (provided the end result conforms to the expected type for the parameter) and correspond to:

- h is a <number> that corresponds to the origin color’s HSL hue after its conversion to sRGB, normalized to a [0, 360) range.

- s is a <percentage> that corresponds to the origin color’s HSL saturation after its conversion to sRGB

- l is a <percentage> that corresponds to the origin color’s HSL lightness after its conversion to sRGB

- alpha is a <percentage> that corresponds to the origin color’s alpha transparency

#### 4.2.3. Relative HWB colors

The grammar of the hwb() function is extended as follows:

    hwb() = hwb([from <color>]? <hue> <percentage> <percentage> [ / <alpha-value> ]? )

When an origin color is present, the following keywords can also be used in this function (provided the end result conforms to the expected type for the parameter) and correspond to:

- h is a <number> that corresponds to the origin color’s HWB hue after its conversion to sRGB

- w is a <percentage> that corresponds to the origin color’s HWB whiteness after its conversion to sRGB

- b is a <percentage> that corresponds to the origin color’s HWB blackness after its conversion to sRGB

- alpha is a <percentage> that corresponds to the origin color’s alpha transparency

#### 4.2.4. Relative Lab colors

The grammar of the lab() function is extended as follows:

    lab() = lab([from <color>]? <percentage> <number> <number> [ / <alpha-value> ]? )

When an origin color is present, the following keywords can also be used in this function (provided the end result conforms to the expected type for the parameter) and correspond to:

- l is a <percentage> that corresponds to the origin color’s CIE Lightness

- a is a <number> that corresponds to the origin color’s CIELab a axis

- b is a <number> that corresponds to the origin color’s CIELab b axis

- alpha is a <percentage> that corresponds to the origin color’s alpha transparency

Multiple ways to adjust the transparency of a base color:

- lab(from var(--mycolor) l a b / 100%) sets the alpha of var(--mycolor) to 100% regardless of what it originally was.

- lab(from var(--mycolor) l a b / calc(alpha * 0.8)) reduces the alpha of var(--mycolor) by 20% of its original value.

- lab(from var(--mycolor) l a b / calc(alpha - 20%)) reduces the alpha of var(--mycolor) by 20% of 100%.

Note that all the adjustments are lossless in the sense that no gamut clipping occurs, since lab() encompasses all visible color. This is not true for the alpha adjustments in the sRGB based functions (such as’rgb()', 'hsl()', or 'hwb()'), which would also convert to sRGB in addition to adjusting the alpha transparency.

#### 4.2.5. Relative LCH colors

The grammar of the lch() function is extended as follows:

    lch() = lch([from <color>]? <percentage> <number> <hue> [ / <alpha-value> ]? )

When an origin color is present, the following keywords can also be used in this function (provided the end result conforms to the expected type for the parameter) and correspond to:

- l is a <percentage> that corresponds to the origin color’s CIE Lightness

- c is a <number> that corresponds to the origin color’s LCH chroma

- h is a <number> that corresponds to the origin color’s LCH hue, normalized to a [0, 360) range.

- alpha is a <percentage> that corresponds to the origin color’s alpha transparency

lch(from peru calc(l * 0.8) c h) produces a color that is 20% darker than peru or lch(62.2532% 54.0114 63.6769), with its chroma and hue left unchanged. The result is lch(49.80256% 54.0114 63.6769)

#### 4.2.6. Relative grayscale colors

The grammar of the gray() function is extended as follows:

    gray() = gray([from <color>]? <number>  [ / <alpha-value> ]? )

When an origin color is present, the following keywords can also be used in this function (provided the end result conforms to the expected type for the parameter) and correspond to:

- l is a <percentage> that corresponds to the origin color’s CIE Lightness

- alpha is a <percentage> that corresponds to the origin color’s alpha transparency

gray(from var(--base-color) l) produces a grayscale version of var(--base-color) with the same lightness.

## Conformance

### Document conventions

Conformance requirements are expressed with a combination of descriptive assertions and RFC 2119 terminology. The key words “MUST”, “MUST NOT”, “REQUIRED”, “SHALL”, “SHALL NOT”, “SHOULD”, “SHOULD NOT”, “RECOMMENDED”, “MAY”, and “OPTIONAL” in the normative parts of this document are to be interpreted as described in RFC 2119. However, for readability, these words do not appear in all uppercase letters in this specification.

All of the text of this specification is normative except sections explicitly marked as non-normative, examples, and notes. [RFC2119]

Examples in this specification are introduced with the words “for example” or are set apart from the normative text with `class="example"`, like this:

This is an example of an informative example.

Informative notes begin with the word “Note” and are set apart from the normative text with `class="note"`, like this:

Note, this is an informative note.

Advisements are normative sections styled to evoke special attention and are set apart from other normative text with `<strong class="advisement">`, like this: UAs MUST provide an accessible alternative.

### Conformance classes

Conformance to this specification is defined for three conformance classes:

style sheet
  A CSS style sheet.
renderer
  A UA that interprets the semantics of a style sheet and renders documents that use them.
authoring tool
  A UA that writes a style sheet.

A style sheet is conformant to this specification if all of its statements that use syntax defined in this module are valid according to the generic CSS grammar and the individual grammars of each feature defined in this module.

A renderer is conformant to this specification if, in addition to interpreting the style sheet as defined by the appropriate specifications, it supports all the features defined by this specification by parsing them correctly and rendering the document accordingly. However, the inability of a UA to correctly render a document due to limitations of the device does not make the UA non-conformant. (For example, a UA is not required to render color on a monochrome monitor.)

An authoring tool is conformant to this specification if it writes style sheets that are syntactically correct according to the generic CSS grammar and the individual grammars of each feature in this module, and meet all other conformance requirements of style sheets as described in this module.

### Requirements for Responsible Implementation of CSS

The following sections define several conformance requirements for implementing CSS responsibly, in a way that promotes interoperability in the present and future.

#### Partial Implementations

So that authors can exploit the forward-compatible parsing rules to assign fallback values, CSS renderers must treat as invalid (and ignore as appropriate) any at-rules, properties, property values, keywords, and other syntactic constructs for which they have no usable level of support. In particular, user agents must not selectively ignore unsupported property values and honor supported values in a single multi-value property declaration: if any value is considered invalid (as unsupported values must be), CSS requires that the entire declaration be ignored.

#### Implementations of Unstable and Proprietary Features

To avoid clashes with future stable CSS features, the CSSWG recommends following best practices for the implementation of unstable features and proprietary extensions to CSS.

#### Implementations of CR-level Features

Once a specification reaches the Candidate Recommendation stage, implementers should release an unprefixed implementation of any CR-level feature they can demonstrate to be correctly implemented according to spec, and should avoid exposing a prefixed variant of that feature.

To establish and maintain the interoperability of CSS across implementations, the CSS Working Group requests that non-experimental CSS renderers submit an implementation report (and, if necessary, the testcases used for that implementation report) to the W3C before releasing an unprefixed implementation of any CSS features. Testcases submitted to W3C are subject to review and correction by the CSS Working Group.

Further information on submitting testcases and implementation reports can be found from on the CSS Working Group’s website at http://www.w3.org/Style/CSS/Test/. Questions should be directed to the public-css-testsuite@w3.org mailing list.

## Index

### Terms defined by this specification

- <alpha-value>, in §4.2.1
- color-adjust(), in §4.1
- color-mix(), in §2
- gray(), in §4.2.6
- hsl(), in §4.2.2
- <hue>, in §4.2.2
- hwb(), in §4.2.3
- lab(), in §4.2.4
- lch(), in §4.2.5
- rgb(), in §4.2.1

https://drafts.csswg.org/css-color-3/#valuea-def-color Referenced in:

- 2. Mixing colors: the color-mix function (2) (3)
- 4.1. Adjusting colors: the color-adjust function (2)
- 4.2. Relative color syntax
- 4.2.1. Relative RGB colors (2)
- 4.2.2. Relative HSL colors
- 4.2.3. Relative HWB colors
- 4.2.4. Relative Lab colors
- 4.2.5. Relative LCH colors
- 4.2.6. Relative grayscale colors

https://drafts.csswg.org/css-color-4/#funcdef-color Referenced in:

- 4.2. Relative color syntax

https://drafts.csswg.org/css-values-3/#angle-value Referenced in:

- 4.2.2. Relative HSL colors

https://drafts.csswg.org/css-values-3/#number-value Referenced in:

- 2. Mixing colors: the color-mix function
- 4.2.1. Relative RGB colors (2)
- 4.2.2. Relative HSL colors (2)
- 4.2.3. Relative HWB colors
- 4.2.4. Relative Lab colors (2) (3) (4)
- 4.2.5. Relative LCH colors (2) (3)
- 4.2.6. Relative grayscale colors

https://drafts.csswg.org/css-values-3/#percentage-value Referenced in:

- 2. Mixing colors: the color-mix function
- 4.2.1. Relative RGB colors (2) (3) (4) (5) (6)
- 4.2.2. Relative HSL colors (2) (3) (4) (5)
- 4.2.3. Relative HWB colors (2) (3) (4) (5)
- 4.2.4. Relative Lab colors (2) (3)
- 4.2.5. Relative LCH colors (2) (3)
- 4.2.6. Relative grayscale colors (2)

https://drafts.csswg.org/css-values-4/#mult-opt Referenced in:

- 2. Mixing colors: the color-mix function (2)
- 4.1. Adjusting colors: the color-adjust function (2)
- 4.2.1. Relative RGB colors (2) (3) (4)
- 4.2.2. Relative HSL colors (2)
- 4.2.3. Relative HWB colors (2)
- 4.2.4. Relative Lab colors (2)
- 4.2.5. Relative LCH colors (2)
- 4.2.6. Relative grayscale colors (2)

https://drafts.csswg.org/css-values-4/#mult-num Referenced in:

- 4.2.1. Relative RGB colors (2)

https://drafts.csswg.org/css-values-4/#comb-one Referenced in:

- 2. Mixing colors: the color-mix function (2)
- 4.2.1. Relative RGB colors (2)
- 4.2.2. Relative HSL colors

https://svgwg.org/svg2-draft/geometry.html#RProperty Referenced in:

- 4.2.1. Relative RGB colors

### Terms defined by reference

- [css-color-3] defines the following terms:
  - <color>
- [css-color-4] defines the following terms:
  - color()
- [css-values-3] defines the following terms:
  - <angle>
  - <number>
  - <percentage>
- [css-values-4] defines the following terms:
  - ?
  - {a}
  - |
- [SVG2] defines the following terms:
  - r

## References

### Normative References

[CSS-COLOR-3]
  Tantek Çelik; Chris Lilley; David Baron. CSS Color Module Level 3. 19 June 2018. REC. URL: https://www.w3.org/TR/css-color-3/
[CSS-COLOR-4]
  Tab Atkins Jr.; Chris Lilley. CSS Color Module Level 4. 5 March 2019. WD. URL: https://www.w3.org/TR/css-color-4/
[CSS-VALUES-3]
  Tab Atkins Jr.; Elika Etemad. CSS Values and Units Module Level 3. 31 January 2019. CR. URL: https://www.w3.org/TR/css-values-3/
[CSS-VALUES-4]
  Tab Atkins Jr.; Elika Etemad. CSS Values and Units Module Level 4. 31 January 2019. WD. URL: https://www.w3.org/TR/css-values-4/
[RFC2119]
  S. Bradner. Key words for use in RFCs to Indicate Requirement Levels. March 1997. Best Current Practice. URL: https://tools.ietf.org/html/rfc2119
[SVG2]
  Amelia Bellamy-Royds; et al. Scalable Vector Graphics (SVG) 2. 4 October 2018. CR. URL: https://www.w3.org/TR/SVG2/

## Issues Index

there are two proposals for color modification (proposal 1, proposal 2). The CSS WG expects that the best aspects of each will be chosen to produce a single eventual solution. <https://github.com/w3c/csswg-drafts/issues/3187> ↵
A future version of this specification may define a relative syntax for color() as well. ↵
#funcdef-rgb Referenced in:

- 4.2.1. Relative RGB colors

#typedef-alpha-value Referenced in:

- 4.2.1. Relative RGB colors (2)
- 4.2.2. Relative HSL colors
- 4.2.3. Relative HWB colors
- 4.2.4. Relative Lab colors
- 4.2.5. Relative LCH colors
- 4.2.6. Relative grayscale colors

#funcdef-hsl Referenced in:

- 4.2.2. Relative HSL colors

#typedef-hue Referenced in:

- 4.2.2. Relative HSL colors
- 4.2.3. Relative HWB colors
- 4.2.5. Relative LCH colors

#funcdef-hwb Referenced in:

- 4.2.3. Relative HWB colors

#funcdef-lab Referenced in:

- 4.2.4. Relative Lab colors

#funcdef-lch Referenced in:

- 2. Mixing colors: the color-mix function (2) (3)
- 4.1. Adjusting colors: the color-adjust function (2)
- 4.2.5. Relative LCH colors

#funcdef-gray Referenced in:

- 4.2.6. Relative grayscale colors